"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExpressApp = void 0;
const http_1 = __importDefault(require("http"));
const path_1 = __importDefault(require("path"));
const express_1 = __importDefault(require("express"));
const cors_1 = __importDefault(require("cors"));
const helmet_1 = __importDefault(require("helmet"));
const logger_1 = __importDefault(require("../logger"));
const utils_1 = require("../utils");
const config_1 = __importDefault(require("../config"));
const RouteManager_1 = __importDefault(require("./RouteManager"));
const PUBLIC_PATH = '/public';
class ExpressApp {
    get isReady() {
        return this._isReady;
    }
    get log() {
        return this._logger;
    }
    get rootDir() {
        return this._rootDir;
    }
    get express() {
        return this._app;
    }
    get server() {
        return this._server;
    }
    constructor(appConfig) {
        this._rootDir = (0, utils_1.getAppRoot)();
        this._logger = logger_1.default;
        this._appConfig = appConfig;
        this._isReady = false;
        this._app = (0, express_1.default)();
        this._server = http_1.default.createServer(this._app);
        this._routes = new RouteManager_1.default(this, this._rootDir);
        this.init();
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.log.info('Loading API configurations...');
                this.log.debug(config_1.default);
                const publicPath = path_1.default.join(this.rootDir, PUBLIC_PATH);
                this.log.info(`Setting public path in ${publicPath}`);
                this._app.use(express_1.default.static(publicPath));
                this.initSecurityMiddlewares();
                this.log.info('Adding healthcheck route');
                this._app.use('/health', (_req, res) => __awaiter(this, void 0, void 0, function* () {
                    const isHealthy = yield this._appConfig.onHealthCheck(this._app);
                    res.status(isHealthy ? 200 : 500).json({ status: isHealthy ? 'ok' : 'no' });
                }));
                this.log.info('Running custom pre-loading...');
                yield this._appConfig.onLoading(this._app);
                this.log.info('Custom pre-loading complete!');
                this.log.info(`Loading Routes in '${this.rootDir}/routes'...`);
                yield this._routes.init();
                this._isReady = true;
                this.log.info('Server is ready...');
                yield this._appConfig.onReady(this._app);
            }
            catch (err) {
                this.log.error('Initialization failed', err);
                process.exit(1);
            }
        });
    }
    initSecurityMiddlewares() {
        var _a, _b;
        this.log.info('Setting up cors...');
        const corsWhitelist = (((_b = (_a = config_1.default === null || config_1.default === void 0 ? void 0 : config_1.default.API) === null || _a === void 0 ? void 0 : _a.SECURITY) === null || _b === void 0 ? void 0 : _b.CORS_ORIGINS) || '*').split(';');
        this.log.debug(corsWhitelist);
        this._app.use((0, cors_1.default)({
            origin: (origin, callback) => {
                if (corsWhitelist[0] === '*') {
                    return callback(null, true);
                }
                if (origin && corsWhitelist.indexOf(origin) !== -1) {
                    return callback(null, true);
                }
                if (!origin) {
                    return callback(null, origin);
                }
                return callback(new Error('Not allowed by CORS'));
            },
        }));
        this.log.info('Applying helmet...');
        this._app.use((0, helmet_1.default)());
    }
    start() {
        if (!this.isReady) {
            throw new Error('SERVER_NOT_READY');
        }
        this._server.listen(this._appConfig.port, () => {
            this.log.info(`listening on port ${this._appConfig.port}!`);
        });
    }
    destroy() {
        return __awaiter(this, void 0, void 0, function* () {
            this._isReady = false;
            yield this._appConfig.onDestroy(this._app);
            this._server.close();
        });
    }
}
exports.ExpressApp = ExpressApp;
//# sourceMappingURL=index.js.map